@@ -57,15 +57,15 @@ AppletInfo *panel_applet_register    (GtkWidget       *applet,
 const char *panel_applet_get_id           (AppletInfo      *info);
 const char *panel_applet_get_id_by_widget (GtkWidget       *widget);
 AppletInfo *panel_applet_get_by_id        (const char      *id);
-AppletInfo *panel_applet_get_by_type      (PanelObjectType  object_type);
+AppletInfo *panel_applet_get_by_type      (PanelObjectType  object_type, GdkScreen *screen);
 
 GSList     *panel_applet_list_applets (void);
 
 void        panel_applet_clean        (AppletInfo    *info);
 
-void panel_applet_queue_applet_to_load (char            *id,
+void panel_applet_queue_applet_to_load (const char      *id,
 					PanelObjectType  type,
-					char            *toplevel_id,
+					const char      *toplevel_id,
 					int              position,
 					gboolean         right_stick,
 					gboolean         locked);
@@ -96,12 +96,19 @@ gboolean    panel_applet_can_freely_move (AppletInfo *applet);
 /* True if the locked flag is writable */
 gboolean    panel_applet_lockable (AppletInfo *applet);
 
+GtkWidget  *panel_applet_create_menu (AppletInfo *info);
+
 void        panel_applet_menu_set_recurse (GtkMenu     *menu,
 					   const gchar *key,
 					   gpointer     data);
 
 gboolean    panel_applet_toggle_locked  (AppletInfo *info);
 
+void panel_applet_position_menu (GtkMenu   *menu,
+				 int       *x,
+				 int       *y,
+				 gboolean  *push_in,
+				 GtkWidget *applet);
 G_END_DECLS
 
 #endif
