@@ -20,8 +20,12 @@
 
 #include "eggaccelerators.h"
 
+#include <stdlib.h>
 #include <string.h>
+#include <gdkconfig.h>
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
+#endif
 #include <gdk/gdkkeysyms.h>
 
 enum
@@ -148,7 +152,7 @@ is_meta (const gchar *string)
 	  (string[2] == 'e' || string[2] == 'E') &&
 	  (string[3] == 't' || string[3] == 'T') &&
 	  (string[4] == 'a' || string[4] == 'A') &&
-	  (string[8] == '>'));
+	  (string[5] == '>'));
 }
 
 static inline gboolean
@@ -160,7 +164,7 @@ is_super (const gchar *string)
 	  (string[3] == 'p' || string[3] == 'P') &&
 	  (string[4] == 'e' || string[4] == 'E') &&
 	  (string[5] == 'r' || string[5] == 'R') &&
-	  (string[8] == '>'));
+	  (string[6] == '>'));
 }
 
 static inline gboolean
@@ -172,7 +176,14 @@ is_hyper (const gchar *string)
 	  (string[3] == 'p' || string[3] == 'P') &&
 	  (string[4] == 'e' || string[4] == 'E') &&
 	  (string[5] == 'r' || string[5] == 'R') &&
-	  (string[8] == '>'));
+	  (string[6] == '>'));
+}
+
+static inline gboolean
+is_keycode (const gchar *string)
+{
+  return ((string[0] == '0') &&
+	  (string[1] == 'x'));
 }
 
 /**
@@ -203,21 +214,26 @@ is_hyper (const gchar *string)
 gboolean
 egg_accelerator_parse_virtual (const gchar            *accelerator,
                                guint                  *accelerator_key,
+			       guint                  *keycode,
                                EggVirtualModifierType *accelerator_mods)
 {
   guint keyval;
   GdkModifierType mods;
   gint len;
   gboolean bad_keyval;
+  gint tmp_keycode;
   
   if (accelerator_key)
     *accelerator_key = 0;
   if (accelerator_mods)
     *accelerator_mods = 0;
+  if (keycode)
+    *keycode = 0;
 
   g_return_val_if_fail (accelerator != NULL, FALSE);
 
   bad_keyval = FALSE;
+  tmp_keycode = 0;
   
   keyval = 0;
   mods = 0;
@@ -314,12 +330,35 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
       else
 	{
           keyval = gdk_keyval_from_name (accelerator);
-          
+
           if (keyval == 0)
-            bad_keyval = TRUE;
-          
+	    {
+	      /* If keyval is 0, than maybe it's a keycode.  Check for 0x## */
+	      if (len >= 4 && is_keycode (accelerator))
+		{
+		  char keystring[5];
+		  gchar *endptr;
+
+		  memcpy (keystring, accelerator, 4);
+		  keystring [4] = '\000';
+
+		  tmp_keycode = strtol (keystring, &endptr, 16);
+
+		  if (endptr == NULL || *endptr != '\000')
+		    {
+		      bad_keyval = TRUE;
+		    }
+		  else
+		    {
+		      /* 0x00 is an invalid keycode too. */
+		      if (tmp_keycode == 0)
+			bad_keyval = TRUE;
+		    }
+		}
+	    }
+
           accelerator += len;
-          len -= len;              
+          len -= len;
 	}
     }
   
@@ -327,8 +366,10 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
     *accelerator_key = gdk_keyval_to_lower (keyval);
   if (accelerator_mods)
     *accelerator_mods = mods;
+  if (keycode)
+    *keycode = tmp_keycode;
 
-  return !bad_keyval && mods != 0;
+  return !bad_keyval;
 }
 
 
@@ -347,6 +388,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
  */
 gchar*
 egg_virtual_accelerator_name (guint                  accelerator_key,
+			      guint		     keycode,
                               EggVirtualModifierType accelerator_mods)
 {
   static const gchar text_release[] = "<Release>";
@@ -366,9 +408,16 @@ egg_virtual_accelerator_name (guint                  accelerator_key,
 
   accelerator_mods &= EGG_VIRTUAL_MODIFIER_MASK;
 
-  keyval_name = gdk_keyval_name (gdk_keyval_to_lower (accelerator_key));
-  if (!keyval_name)
-    keyval_name = "";
+  if (!accelerator_key)
+    {
+      keyval_name = g_strdup_printf ("0x%02x", keycode);
+    }
+  else
+    {
+      keyval_name = gdk_keyval_name (gdk_keyval_to_lower (accelerator_key));
+      if (!keyval_name)
+        keyval_name = "";
+    }
 
   l = 0;
   if (accelerator_mods & EGG_VIRTUAL_RELEASE_MASK)
@@ -541,6 +590,7 @@ static void
 reload_modmap (GdkKeymap *keymap,
                EggModmap *modmap)
 {
+#ifdef GDK_WINDOWING_X11
   XModifierKeymap *xmodmap;
   int map_size;
   int i;
@@ -621,6 +671,9 @@ reload_modmap (GdkKeymap *keymap,
   modmap->mapping[EGG_MODMAP_ENTRY_MOD5] |= EGG_VIRTUAL_MOD5_MASK;
   
   XFreeModifiermap (xmodmap);
+#else
+  g_error ("Not yet implemented for Win32: eggaccelerators.c:reload_modmap()");
+#endif
 }
 
 const EggModmap*
