@@ -1,7 +1,4 @@
 #include "config.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -21,12 +18,12 @@ GSList *panel_applet_list_applets (void) { return NULL; }
 
 static int dialogs = 0;
 static gboolean create_new = FALSE;
+static char **desktops = NULL;
 
-static struct poptOption options [] = {
-	{ "create-new", '\0', POPT_ARG_NONE,
-	  &create_new, 0, N_("Create new file in the given directory"), NULL },
-        POPT_AUTOHELP
-	{ NULL, 0, 0, NULL, 0}
+static const GOptionEntry options[] = {
+	{ "create-new", 0, 0, G_OPTION_ARG_NONE, &create_new, N_("Create new file in the given directory"), NULL },
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &desktops, NULL, "[FILE...]" }, //FIXME 2.16 mark for translation
+	{ NULL }
 };
 
 
@@ -73,9 +70,8 @@ get_uri (const char *arg)
 int
 main (int argc, char * argv[])
 {
-	poptContext ctx;
+	GOptionContext *context;
 	GnomeProgram *program;
-	char **desktops;
 	int i;
 	GnomeVFSFileInfo *info;
 
@@ -83,22 +79,21 @@ main (int argc, char * argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	context = g_option_context_new (""); //FIXME 2.16 "- Edit .desktop files"
+
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	program = gnome_program_init ("gnome-desktop-item-edit", VERSION,
 				      LIBGNOMEUI_MODULE,
 				      argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      NULL);
-	g_object_get (G_OBJECT (program),
-		      GNOME_PARAM_POPT_CONTEXT, &ctx,
-		      NULL);
 
 	gtk_window_set_default_icon_name ("launcher-program");
 
-	desktops = (char **)poptGetArgs (ctx);
-
 	if (desktops == NULL ||
 	    desktops[0] == NULL) {
-		fprintf (stderr, "gnome-desktop-item-edit: no file to edit\n");
+		g_printerr ("gnome-desktop-item-edit: no file to edit\n");
 		return 0;
 	}
 
@@ -146,9 +141,9 @@ main (int argc, char * argv[])
 				g_free (dirname);
 				
 			} else {
-				fprintf (stderr, "gnome-desktop-item-edit: %s "
-					 "doesn't seem like a desktop item\n",
-					 uri);
+				g_printerr ("gnome-desktop-item-edit: %s "
+					    "doesn't seem like a desktop "
+					    "item\n", uri);
 			}
 
 		} else if (g_str_has_suffix (desktops [i], ".directory")
@@ -172,8 +167,8 @@ main (int argc, char * argv[])
 			g_free (dirname);
 
 		} else {
-			fprintf (stderr, "gnome-desktop-item-edit: %s does "
-				 "not exist\n", uri);
+			g_printerr ("gnome-desktop-item-edit: %s does "
+				    "not exist\n", uri);
 		}
 
 		if (dlg != NULL) {
@@ -191,5 +186,7 @@ main (int argc, char * argv[])
 	if (dialogs > 0)
 		gtk_main ();
 
+	g_object_unref (program);
+
         return 0;
 }
