@@ -28,13 +28,21 @@ typedef struct {
 	GnomeDesktopItem *revert_ditem;
 
 	GtkWidget         *prop_dialog;
+	GSList 		  *error_dialogs;
 
 	gulong             destroy_handler;
+
+	/* If we can't hoard, and it's not hoarded
+	   already, then we can't write this launcher */
+	gboolean           non_writable;
 } Launcher;
 
 void panel_launcher_create           (PanelToplevel *toplevel,
 				      int            position,
 				      const char    *location);
+void panel_launcher_create_with_id   (const char    *toplevel_id,
+				      int            position,
+				      const char    *location);
 void panel_launcher_create_copy      (PanelToplevel *toplevel,
 				      int            position,
 				      const char    *location);
@@ -46,10 +54,14 @@ void panel_launcher_create_from_info (PanelToplevel *toplevel,
 				      const char    *comment,
 				      const char    *icon);
 
+void		launcher_launch    		(Launcher  *launcher,
+						 GtkWidget *widget);
+
 void		launcher_properties		(Launcher  *launcher,
 						 GdkScreen *screen);
 
 void            launcher_load_from_gconf        (PanelWidget *panel_widget,
+						 gboolean     locked,
 						 gint         position,
 						 const char  *id);
 
@@ -62,11 +74,12 @@ void		ask_about_launcher		(const char *file,
 
 Launcher *	find_launcher			(const char *path);
 
-void            launcher_show_help              (Launcher  *launcher,
-						 GdkScreen *screen);
-
 void            launcher_properties_destroy     (Launcher *launcher);
 
+void            panel_launcher_set_dnd_enabled  (Launcher *launcher,
+						 gboolean  dnd_enabled);
+
+
 G_END_DECLS
 
 #endif
