@@ -47,6 +47,7 @@
 #include "panel-lockdown.h"
 #include "panel-logout.h"
 #include "panel-compatibility.h"
+#include "panel-gdm.h"
 
 #define PANEL_ACTION_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_ACTION_BUTTON, PanelActionButtonPrivate))
 
@@ -165,6 +166,13 @@ panel_action_shutdown (GtkWidget *widget)
 			  gtk_get_current_event_time ());
 }
 
+static gboolean
+panel_action_shutdown_is_disabled (void)
+{
+	return (panel_lockdown_get_disable_log_out() ||
+		!gdm_supports_logout_action (GDM_LOGOUT_ACTION_SHUTDOWN));
+}
+
 /* Run Application
  */
 static void
@@ -295,8 +303,8 @@ static PanelAction actions [] = {
 		/* when changing one of those two strings, don't forget to
 		 * update the ones in panel-menu-items.c (look for
 		 * panel:showusername|1) */
-		N_("Log Out"),
-		N_("Log out of this session to log in as a different user or to shut down the computer"),
+		N_("Log Out..."),
+		N_("Log out of this session to log in as a different user"),
 		"gospanel-20",
 		"ACTION:logout:NEW",
 		panel_action_logout, NULL, NULL,
@@ -343,12 +351,12 @@ static PanelAction actions [] = {
 	{
 		PANEL_ACTION_SHUTDOWN,
 		"gnome-shutdown",
-		N_("Shut Down"),
+		N_("Shut Down..."),
 		N_("Shut down the computer"),
 		"gospanel-20",
 		"ACTION:shutdown:NEW",
 		panel_action_shutdown, NULL, NULL,
-		panel_lockdown_get_disable_log_out
+		panel_action_shutdown_is_disabled
 	},
 	/* deprecated actions */
 	{
