@@ -402,12 +402,18 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 		}
 
 		if (!label) {
-			if (gnome_vfs_uri_is_local (bookmark->uri)) {
+			if (!strcmp (gnome_vfs_uri_get_scheme (bookmark->uri),
+							       "file")) {
 				char *buffer;
 
 				buffer = gnome_vfs_get_local_path_from_uri (full_uri);
 				label = g_filename_display_basename (buffer);
 				g_free (buffer);
+			} else if (gnome_vfs_uri_is_local (bookmark->uri)) {
+				/* local non-file methods, such as burn:,
+				 * font:, etc. */
+				//FIXME 2.16: we should do the same than in nautilus (see nautilus_file_get_display_name_nocopy() and nautilus_get_vfs_method_display_name()
+				label = gnome_vfs_format_uri_for_display (full_uri);
 			} else {
 				const char *path;
 				const char *hostname;
@@ -418,7 +424,7 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 				if (!path || !path[0])
 					displayname = g_strdup ("/");
 				else
-					displayname = g_filename_display_basename (path);
+					displayname = gnome_vfs_unescape_string_for_display (path);
 				if (hostname) {
 					/* Translators: the first string is a
 					 * path and the second string is a
@@ -452,26 +458,6 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 	g_slist_free (add_bookmarks);
 }
 
-static gint
-panel_place_menu_item_sort_volume (GnomeVFSVolume *volume_a,
-				   GnomeVFSVolume *volume_b) {
-	char *display_a;
-	char *display_b;
-	int   retval;
-
-	g_return_val_if_fail (volume_a && volume_b, 0);
-
-	display_a = gnome_vfs_volume_get_display_name (volume_a);
-	display_b = gnome_vfs_volume_get_display_name (volume_b);
-
-	retval = g_utf8_collate (display_a, display_b);
-
-	g_free (display_a);
-	g_free (display_b);
-
-	return retval;
-}
-
 static void
 panel_place_menu_item_append_volumes (GtkWidget *menu,
 				      gboolean   connected_volumes)
@@ -504,11 +490,8 @@ panel_place_menu_item_append_volumes (GtkWidget *menu,
 		}
 	}
 
-	if (connected_volumes)
-		add_volumes = g_slist_sort (add_volumes,
-					    (GCompareFunc) panel_place_menu_item_sort_volume);
-	else
-		add_volumes = g_slist_reverse (add_volumes);
+	add_volumes = g_slist_sort (add_volumes,
+				    (GCompareFunc) gnome_vfs_volume_compare);
 
 	if (g_slist_length (add_volumes) <= MAX_ITEMS_OR_SUBMENU) {
 		add_menu = menu;
@@ -704,11 +687,17 @@ panel_desktop_menu_item_append_menu (GtkWidget            *menu,
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
 
 	if (children != NULL) {
-		while (children->next != NULL)
-			children = children->next;
-		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
+		GList *child;
+
+		child = children;
+		while (child->next != NULL)
+			child = child->next;
+
+		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (child->data));
 	}
 
+	g_list_free (children);
+
 	if (add_separator)
 		add_menu_separator (menu);
 
@@ -1092,12 +1081,12 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 		 * panel-action-button.c */
 		/* Translators: this string is used ONLY if you translated
 		 * "panel:showusername|1" to "1" */
-		label = g_strdup_printf (_("Log Out %s"), g_get_user_name ());
+		label = g_strdup_printf (_("Log Out %s..."),
+					 g_get_user_name ());
 		/* Translators: this string is used ONLY if you translated
 		 * "panel:showusername|1" to "1" */
 		tooltip = g_strdup_printf (_("Log out %s of this session to "
-					     "log in as a different user or "
-					     "to shut down your computer"),
+					     "log in as a different user"),
 					   user_name);
 	} else {
 		label   = NULL;
