@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  * GNOME panel utils
  * (C) 1997, 1998, 1999, 2000 The Free Software Foundation
  * Copyright 2000 Helix Code, Inc.
@@ -12,39 +13,52 @@
  */
 
 #include <config.h>
-#include <string.h>
-#include <glib.h>
-#include <sys/types.h>
-#include <fcntl.h>
-#include <dirent.h>
-
-/* includes for the battery stuff */
-#include <signal.h>
-#include <limits.h>
-#include <unistd.h>
-#include <sys/wait.h>
-
-#include <libgnomeui/libgnomeui.h>
-#include <libgnome/libgnome.h>
-
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
 
 #include "panel-util.h"
 
+#include <string.h>
+#include <unistd.h>
+#include <sys/types.h>
+
+#include <glib/gi18n.h>
+#include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-help.h>
+
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+
 #include "applet.h"
 #include "nothing.h"
-#include "panel.h"
-#include "panel-a11y.h"
-#include "egg-screen-exec.h"
-#include "egg-screen-url.h"
-#include "egg-screen-help.h"
 #include "xstuff.h"
 #include "panel-globals.h"
 #include "launcher.h"
 
+void
+panel_launch_desktop_file (const char  *desktop_file,
+			   const char  *fallback_exec,
+			   GdkScreen   *screen,
+			   GError     **error)
+{
+	GnomeDesktopItem *ditem;
+
+	ditem = gnome_desktop_item_new_from_basename (desktop_file, 0, NULL);
+	if (ditem == NULL) {
+		char *argv [2] = {(char *)fallback_exec, NULL};
+
+		gdk_spawn_on_screen (screen, NULL, argv, NULL,
+				     G_SPAWN_SEARCH_PATH,
+				     NULL, NULL, NULL, error);
+		return;
+	}
+
+	panel_ditem_launch (ditem, NULL, 0, screen, error);
+
+	gnome_desktop_item_unref (ditem);
+}
+
 int
-panel_ditem_launch (const GnomeDesktopItem       *item,
+panel_ditem_launch (GnomeDesktopItem             *item,
 		    GList                        *file_list,
 		    GnomeDesktopItemLaunchFlags   flags,
 		    GdkScreen                    *screen,
@@ -54,6 +68,8 @@ panel_ditem_launch (const GnomeDesktopItem       *item,
 
 	workspace = xstuff_get_current_workspace (screen);
 
+	gnome_desktop_item_set_launch_time (item, gtk_get_current_event_time ());
+
 	return gnome_desktop_item_launch_on_screen (
 			item, file_list, flags, screen, workspace, error);
 }
@@ -65,11 +81,12 @@ panel_show_help (GdkScreen  *screen,
 {
 	GError *error = NULL;
 
-	if (!egg_help_display_desktop_on_screen (NULL, "user-guide", doc_name, linkid, screen, &error)) {
+	if (!gnome_help_display_desktop_on_screen (NULL, "user-guide", doc_name, linkid, screen, &error)) {
 		panel_error_dialog (
 			screen,
 			"cannot_show_help",
-			_("Cannot display help document"),
+			TRUE,
+			_("Could not display help document"),
 			"%s",
 			error != NULL ? error->message : "");
 
@@ -208,11 +225,44 @@ panel_g_list_resort_item(GList *list, gpointer data, GCompareFunc func)
 	return list;
 }
 
+GSList *
+panel_g_slist_make_unique (GSList       *list,
+			   GCompareFunc  compare,
+			   gboolean      free_data)
+{
+	GSList *sorted, *l;
+
+	g_assert (compare != NULL);
+
+	if (!list)
+		return NULL;
+
+	sorted = g_slist_copy (list);
+	sorted = g_slist_sort (sorted, compare);
+
+	for (l = sorted; l; l = l->next) {
+		GSList *next;
+
+		next = l->next;
+		if (l->data && next && next->data)
+			if (!compare (l->data, next->data)) {
+				list = g_slist_remove (list, l->data);
+				if (free_data)
+					g_free (l->data);
+			}
+	}
+
+	g_slist_free (sorted);
+
+	return list;
+}
+
 static GtkWidget *
 panel_dialog (GdkScreen  *screen,
 	      int         type,
 	      const char *class,
-	      const char *str)
+	      const char *str,
+	      gboolean    auto_destroy)
 {
 	GtkWidget *dialog;
 
@@ -228,15 +278,13 @@ panel_dialog (GdkScreen  *screen,
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog), class, "Panel");
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
 	gtk_widget_show_all (dialog);
 
-	/* FIXME: this is ugly and makes it bad to run gtk_dialog_run
-	 * after this function */
-	g_signal_connect_swapped (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (dialog));
+	if (auto_destroy)
+		g_signal_connect_swapped (G_OBJECT (dialog), "response",
+					  G_CALLBACK (gtk_widget_destroy),
+					  G_OBJECT (dialog));
 
 	return dialog;
 }
@@ -244,6 +292,7 @@ panel_dialog (GdkScreen  *screen,
 GtkWidget *
 panel_error_dialog (GdkScreen  *screen,
 		    const char *class,
+		    gboolean    auto_destroy,
 		    const char *primary_format,
 		    const char *secondary_format,
 		    ...)
@@ -276,7 +325,7 @@ panel_error_dialog (GdkScreen  *screen,
 			g_free (format);
 	}
 
-	w = panel_dialog (screen, GTK_MESSAGE_ERROR, class, s);
+	w = panel_dialog (screen, GTK_MESSAGE_ERROR, class, s, auto_destroy);
 	g_free (s);
 	return w;
 }
@@ -340,127 +389,6 @@ panel_pop_window_busy (GtkWidget *window)
 	}
 }
 
-/*
- * GnomeVFS reading utils, that look like the libc buffered io stuff
- */
-struct _ReadBuf {
-	GnomeVFSHandle *handle;
-	char *uri;
-	gboolean eof;
-	char buf[BUFSIZ];
-	gsize size;
-	gsize pos;
-};
-
-static int
-readbuf_getc (ReadBuf *rb)
-{
-	if (rb->eof)
-		return EOF;
-
-	if (rb->size == 0 ||
-	    rb->pos == rb->size) {
-		GnomeVFSFileSize bytes_read;
-		/* FIXME: handle other errors */
-		if (gnome_vfs_read (rb->handle,
-				    rb->buf,
-				    BUFSIZ,
-				    &bytes_read) != GNOME_VFS_OK) {
-			rb->eof = TRUE;
-			return EOF;
-		}
-		rb->size = bytes_read;
-		rb->pos = 0;
-
-		if (rb->size == 0) {
-			rb->eof = TRUE;
-			return EOF;
-		}
-	}
-
-	return (int)rb->buf[rb->pos++];
-}
-
-/* Note, does not include the trailing \n */
-char *
-readbuf_gets (char *buf, gsize bufsize, ReadBuf *rb)
-{
-	int c;
-	gsize pos;
-
-	g_return_val_if_fail (rb != NULL, NULL);
-
-	pos = 0;
-	buf[0] = '\0';
-
-	do {
-		c = readbuf_getc (rb);
-		if (c == EOF ||
-		    c == '\n')
-			break;
-		buf[pos++] = c;
-	} while (pos < bufsize-1);
-
-	if (c == EOF &&
-	    pos == 0)
-		return NULL;
-
-	buf[pos++] = '\0';
-
-	return buf;
-}
-
-ReadBuf *
-readbuf_open (const char *uri)
-{
-	GnomeVFSHandle *handle;
-	ReadBuf *rb;
-
-	g_return_val_if_fail (uri != NULL, NULL);
-
-	if (gnome_vfs_open (&handle, uri,
-			    GNOME_VFS_OPEN_READ) != GNOME_VFS_OK)
-		return NULL;
-
-	rb = g_new0 (ReadBuf, 1);
-	rb->handle = handle;
-	rb->uri = g_strdup (uri);
-	rb->eof = FALSE;
-	rb->size = 0;
-	rb->pos = 0;
-
-	return rb;
-}
-
-/* unused for now */
-gboolean
-readbuf_rewind (ReadBuf *rb)
-{
-	if (gnome_vfs_seek (rb->handle,
-			    GNOME_VFS_SEEK_START, 0) == GNOME_VFS_OK)
-		return TRUE;
-
-	gnome_vfs_close (rb->handle);
-	rb->handle = NULL;
-	if (gnome_vfs_open (&rb->handle, rb->uri,
-			    GNOME_VFS_OPEN_READ) == GNOME_VFS_OK)
-		return TRUE;
-
-	return FALSE;
-}
-
-void
-readbuf_close (ReadBuf *rb)
-{
-	if (rb->handle != NULL)
-		gnome_vfs_close (rb->handle);
-	rb->handle = NULL;
-	g_free (rb->uri);
-	rb->uri = NULL;
-
-	g_free (rb);
-}
-
 gboolean
 panel_is_program_in_path (const char *program)
 {
@@ -473,44 +401,6 @@ panel_is_program_in_path (const char *program)
 	}
 }
 
-char *
-panel_pixmap_discovery (const char *name, gboolean fallback)
-{
-	char *pixmap;
-	pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
-					    name,
-					    TRUE /* only_if_exists */,
-					    NULL /* ret_locations */);
-	if (pixmap == NULL)
-		pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, 
-						    name,
-						    TRUE /* only_if_exists */,
-						    NULL /* ret_locations */);
-	/* just as a fallback return something */
-	if (pixmap == NULL && fallback)
-		pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
-						    name,
-						    FALSE /* only_if_exists */,
-						    NULL /* ret_locations */);
-	return pixmap;
-}
-
-/* stolen from gtk */
-void
-panel_signal_connect_object_while_alive (gpointer    object,
-					 const char *signal,
-					 GCallback   func,
-					 gpointer    alive_object)
-{
-	g_return_if_fail (G_IS_OBJECT (object));
-	g_return_if_fail (G_IS_OBJECT (alive_object));
-  
-	g_signal_connect_closure_by_id (object,
-					g_signal_lookup (signal, G_OBJECT_TYPE (object)), 0,
-					g_cclosure_new_object_swap (func, G_OBJECT (alive_object)),
-					FALSE);
-}
-
 static gboolean
 panel_ensure_dir (const char *dirname)
 {
@@ -551,17 +441,6 @@ panel_ensure_dir (const char *dirname)
 	return TRUE;
 }
 
-void
-panel_g_slist_deep_free	(GSList *list)
-{
-	GSList *li;
-	for (li = list; li != NULL; li = li->next) {
-		g_free (li->data);
-		li->data = NULL;
-	}
-	g_slist_free (list);
-}
-
 static gboolean
 internal_panel_is_uri_writable (const char *uri, gboolean recurse)
 {
@@ -672,6 +551,98 @@ panel_uri_exists (const char *uri)
 	return ret;
 }
 
+char *
+panel_find_icon (GtkIconTheme  *icon_theme,
+		 const char    *icon_name,
+		 gint           size)
+{
+	GtkIconInfo *info;
+	char        *retval;
+	char        *icon_no_extension;
+	char        *p;
+
+	if (icon_name == NULL || strcmp (icon_name, "") == 0)
+		return NULL;
+
+	if (g_path_is_absolute (icon_name)) {
+		if (g_file_test (icon_name, G_FILE_TEST_EXISTS)) {
+			return g_strdup (icon_name);
+		} else {
+			char *basename;
+
+			basename = g_path_get_basename (icon_name);
+			retval = panel_find_icon (icon_theme, basename,
+						  size);
+			g_free (basename);
+
+			return retval;
+		}
+	}
+
+	/* This is needed because some .desktop files have an icon name *and*
+	 * an extension as icon */
+	icon_no_extension = g_strdup (icon_name);
+	p = strrchr (icon_no_extension, '.');
+	if (p &&
+	    (strcmp (p, ".png") == 0 ||
+	     strcmp (p, ".xpm") == 0 ||
+	     strcmp (p, ".svg") == 0)) {
+	    *p = 0;
+	}
+
+	info = gtk_icon_theme_lookup_icon (icon_theme, icon_no_extension,
+					   size, 0);
+
+	g_free (icon_no_extension);
+
+	if (info) {
+		retval = g_strdup (gtk_icon_info_get_filename (info));
+		gtk_icon_info_free (info);
+	} else
+		retval = NULL;
+
+	return retval;
+}
+
+GdkPixbuf *
+panel_load_icon (GtkIconTheme  *icon_theme,
+		 const char    *icon_name,
+		 int            size,
+		 int            desired_width,
+		 int            desired_height,
+		 char         **error_msg)
+{
+	GdkPixbuf *retval;
+	char      *file;
+	GError    *error;
+
+	g_return_val_if_fail (error_msg == NULL || *error_msg == NULL, NULL);
+
+	file = panel_find_icon (icon_theme, icon_name, size);
+	if (!file) {
+		if (error_msg)
+			*error_msg = g_strdup_printf (_("Icon '%s' not found"),
+						      icon_name);
+
+		return NULL;
+	}
+
+	error = NULL;
+	retval = gdk_pixbuf_new_from_file_at_size (file,
+						   desired_width,
+						   desired_height,
+						   &error);
+	if (error) {
+		if (error_msg)
+			*error_msg = g_strdup (error->message);
+		g_error_free (error);
+	}
+
+	g_free (file);
+
+	return retval;
+}
+
 GdkPixbuf *
 missing_pixbuf (int size)
 {
@@ -705,21 +676,88 @@ missing_pixbuf (int size)
 	return pb;
 }
 
+static char *
+panel_lock_screen_action_get_command (const char *action)
+{
+	char    *command          = NULL;
+	gboolean use_gscreensaver = FALSE;
+
+	if (panel_is_program_in_path ("gnome-screensaver-command")
+	    && panel_is_program_in_path ("gnome-screensaver-preferences"))
+		use_gscreensaver = TRUE;
+	else if (!panel_is_program_in_path ("xscreensaver-command"))
+		return NULL;
+
+	if (strcmp (action, "prefs") == 0) {
+		if (use_gscreensaver) {
+			command = g_strdup ("gnome-screensaver-preferences");
+		} else if (panel_is_program_in_path ("xscreensaver-demo")) {
+			command = g_strdup ("xscreensaver-demo");
+		} else {
+			command = NULL;
+		}
+	} else if (strcmp (action, "activate") == 0
+		   || strcmp (action, "lock") == 0) {
+		if (use_gscreensaver) {
+			command = g_strdup_printf ("gnome-screensaver-command --%s", action);
+		} else {
+			command = g_strdup_printf ("xscreensaver-command -%s", action);
+		}
+	}
+
+	return command;
+}
+
+gboolean
+panel_lock_screen_action_available (const char *action)
+{
+	char    *command;
+	gboolean enabled = FALSE;
+
+	g_return_val_if_fail (action != NULL, FALSE);
+
+	command = panel_lock_screen_action_get_command (action);
+	if (command)
+		enabled = TRUE;
+
+	g_free (command);
+
+	return enabled;
+}
+
+void
+panel_lock_screen_action (GdkScreen  *screen,
+			  const char *action)
+{
+	GError  *error            = NULL;
+	char    *command          = NULL;
+
+	g_return_if_fail (GDK_IS_SCREEN (screen));
+	g_return_if_fail (action != NULL);
+
+	command = panel_lock_screen_action_get_command (action);
+
+	if (!command)
+		return;
+
+	if (!gdk_spawn_command_line_on_screen (screen, command, &error)) {
+		panel_error_dialog (screen,
+				    "cannot_exec_screensaver",
+				    TRUE,
+				    _("Could not execute '%s'"),
+				    "%s",
+				    command,
+				    error->message);
+		g_error_free (error);
+	}
+
+	g_free (command);
+}
+
 void
 panel_lock_screen (GdkScreen *screen)
 {
-	char *argv[3] = {"xscreensaver-command", "-lock", NULL};
-
-	if (!screen)
-		screen = gdk_screen_get_default ();
-
-	if (egg_screen_execute_async (screen, g_get_home_dir (), 2, argv) < 0)
-		panel_error_dialog (screen,
-			"cannot_exec_xscreensaver",
-			 _("Cannot execute %s"),
-			 _("%s - command not found"),
-			 "xscreensaver",
-			 "xscreensaver");
+	panel_lock_screen_action (screen, "lock");
 }
 
 
@@ -749,8 +787,8 @@ panel_make_full_path (const char *dir,
 }
 
 char *
-panel_make_unique_path (const char *dir,
-			const char *suffix)
+panel_make_unique_uri (const char *dir,
+		       const char *suffix)
 {
 #define NUM_OF_WORDS 12
 	char *words[] = {
@@ -767,7 +805,8 @@ panel_make_unique_path (const char *dir,
 		"curly",
 		"moe",
 		NULL};
-	char     *retval = NULL;
+	char     *uri;
+	char     *path = NULL;
 	gboolean  exists = TRUE;
 
 	while (exists) {
@@ -783,13 +822,165 @@ panel_make_unique_path (const char *dir,
 					    (guint) rnd,
 					    suffix);
 
-		g_free (retval);
-		retval = panel_make_full_path (dir, filename);
-		exists = g_file_test (retval, G_FILE_TEST_EXISTS);
+		g_free (path);
+		path = panel_make_full_path (dir, filename);
+		exists = g_file_test (path, G_FILE_TEST_EXISTS);
 		g_free (filename);
 	}
 
-	return retval;
+	uri = gnome_vfs_get_uri_from_local_path (path);
+	g_free (path);
+
+	return uri;
 
 #undef NUM_OF_WORDS
 }
+
+static char *
+lookup_in_data_dir (const char *basename,
+                    const char *data_dir)
+{
+	char *path;
+
+	path = g_build_filename (data_dir, basename, NULL);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+		return NULL;
+	}
+
+	return path;
+}
+
+char *
+panel_lookup_in_data_dirs (const char *basename)
+{
+	const char * const *system_data_dirs;
+	const char          *user_data_dir;
+	char                *retval;
+	int                  i;
+
+	user_data_dir    = g_get_user_data_dir ();
+	system_data_dirs = g_get_system_data_dirs ();
+
+	if ((retval = lookup_in_data_dir (basename, user_data_dir)))
+		return retval;
+
+	for (i = 0; system_data_dirs[i]; i++)
+		if ((retval = lookup_in_data_dir (basename, system_data_dirs[i])))
+			return retval;
+
+	return NULL;
+}
+
+/* Stolen from evolution-data-server/libedataserver/e-util.c:
+ * e_util_unicode_get_utf8()
+ * e_util_utf8_strstrcase() */
+static char *
+panel_util_unicode_get_utf8 (const char *text, gunichar *out)
+{
+	*out = g_utf8_get_char (text);
+	return (*out == (gunichar)-1) ? NULL : g_utf8_next_char (text);
+}
+
+const char *
+panel_util_utf8_strstrcase (const char *haystack, const char *needle)
+{
+	gunichar *nuni;
+	gunichar unival;
+	gint nlen;
+	const char *o, *p;
+
+	if (haystack == NULL) return NULL;
+	if (needle == NULL) return NULL;
+	if (strlen (needle) == 0) return haystack;
+	if (strlen (haystack) == 0) return NULL;
+
+	nuni = g_alloca (sizeof (gunichar) * strlen (needle));
+
+	nlen = 0;
+	for (p = panel_util_unicode_get_utf8 (needle, &unival); p && unival; p = panel_util_unicode_get_utf8 (p, &unival)) {
+		nuni[nlen++] = g_unichar_tolower (unival);
+	}
+	/* NULL means there was illegal utf-8 sequence */
+	if (!p) return NULL;
+
+	o = haystack;
+	for (p = panel_util_unicode_get_utf8 (o, &unival); p && unival; p = panel_util_unicode_get_utf8 (p, &unival)) {
+		gint sc;
+		sc = g_unichar_tolower (unival);
+		/* We have valid stripped char */
+		if (sc == nuni[0]) {
+			const char *q = p;
+			gint npos = 1;
+			while (npos < nlen) {
+				q = panel_util_unicode_get_utf8 (q, &unival);
+				if (!q || !unival) return NULL;
+				sc = g_unichar_tolower (unival);
+				if (sc != nuni[npos]) break;
+				npos++;
+			}
+			if (npos == nlen) {
+				return o;
+			}
+		}
+		o = p;
+	}
+
+	return NULL;
+}
+
+GdkPixbuf *
+panel_util_cairo_rgbdata_to_pixbuf (unsigned char *data,
+				    int            width,
+				    int            height)
+{
+	GdkPixbuf     *retval;
+	unsigned char *dstptr;
+	unsigned char *srcptr;
+	int            align;
+
+	g_assert (width > 0 && height > 0);
+
+	if (!data)
+		return NULL;
+
+	retval = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, width, height);
+	if (!retval)
+		return NULL;
+
+	dstptr = gdk_pixbuf_get_pixels (retval);
+	srcptr = data;
+	align  = gdk_pixbuf_get_rowstride (retval) - (width * 3);
+
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+/* cairo == 00RRGGBB */
+#define CAIRO_RED 2
+#define CAIRO_GREEN 1
+#define CAIRO_BLUE 0
+#else
+/* cairo == BBGGRR00 */
+#define CAIRO_RED 1
+#define CAIRO_GREEN 2
+#define CAIRO_BLUE 3
+#endif
+
+	while (height--) {
+		int x = width;
+		while (x--) {
+			/* pixbuf == BBGGRR */
+			dstptr[0] = srcptr[CAIRO_RED];
+			dstptr[1] = srcptr[CAIRO_GREEN];
+			dstptr[2] = srcptr[CAIRO_BLUE];
+
+			dstptr += 3;
+			srcptr += 4;
+		}
+
+		dstptr += align;
+	}
+#undef CAIRO_RED
+#undef CAIRO_GREEN
+#undef CAIRO_BLUE
+
+	return retval;
+}
